// Copyright (c) 2014, the Dart project authors.  Please see the AUTHORS file
// for details. All rights reserved. Use of this source code is governed by a
// BSD-style license that can be found in the LICENSE file.

/**
 * `DartSdk` implementations that use `dart:io` to retrieve the SDK contents.
 */
library compiler_unsupported.sdk_io;

import 'dart:async';
import 'dart:convert' show UTF8;
import 'dart:html';

import 'package:compiler_unsupported/version.dart' as sdk_version;

import 'sdk.dart';
export 'sdk.dart';

/**
 * This `DartSdk` implementation reads the SDK resources encoded into the
 * `compiler_unsupported` package as part of a build step. This does not rely
 * on being able to locate an SDK on disk.
 */
class DartSdkHtml implements DartSdkAsync {
  static final _pkgPath = 'packages/compiler_unsupported/sdk';

  Map<String, String> _cache = {};
  final List<String> _paths;

  //ZLibCodec _zlib = new ZLibCodec();

  Future<DartSdkHtml> createSdk() {
    // TODO: read the files.json file

  }

  DartSdkHtml._(this._paths);

  String get version => sdk_version.versionLong;

  String get location => Uri.base.toString();

  List<String> getAllSourcePaths() => _paths;

  Future<String> getSourceForPath(String path) {
    if (_cache.containsKey(path)) {
      return new Future.value(_cache[path]);
    }

    if (path.length < 4) return null;

    // Remove the `/lib` dir.
    var p = _pkgPath + path.substring(4) + '_';

    File file = new File(p);

    if (file.existsSync()) {
      List bytes = file.readAsBytesSync();
      bytes = _zlib.decode(bytes);
      _cache[path] = UTF8.decode(bytes);
      return new Future.value(_cache[path]);
    }

    return new Future.value(null);
  }
}
